package controler;

import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import java.time.LocalTime;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileNameExtensionFilter;

import model.Alarm;
import model.AlarmList;
import model.FileType;
import model.Time;
import view.AboutDialog;
import view.AnalogicAlarmView;
import view.Digital12AlarmView;
import view.AlarmDialog;
import view.AlarmListFrame;
import view.Digital24AlarmView;

public class Controler
extends Thread
implements WindowListener
{
	private AlarmListFrame frame;
	private AlarmList alarms;
	private int alarmViewCount = 0;
	public Controler() {
		frame = null;
		alarms = null;
  	}
	public void setView(AlarmListFrame f) {
		frame = f;
	}
	public void setModel(AlarmList l) {
		alarms = l;
	}
	public void about() {
		(AboutDialog.getInstance()).display();
	}
	public void viewAlarm(Alarm a) {
		switch (alarmViewCount % 3) {
		case 0:
			new Digital24AlarmView(a, frame).setVisible(true);
			break;
		case 1:
			new Digital12AlarmView(a, frame).setVisible(true);
			break;
		case 2:
			new AnalogicAlarmView(a, frame).setVisible(true);
			break;
		}
		alarmViewCount++;
	}
	public void newAlarm() {
		try {
			Alarm alarm = new Alarm("nouvelle", "12:00");
			alarms.add(alarm);
		} catch(Exception e) { };
		alarms.sort();
		frame.updateAlarmList();
		frame.disableEditor();
		alarms.setChanged();
        frame.updateTitle();
	}
	public void loadAlarms() {
        JFileChooser fileChooser = new JFileChooser();
        FileType ft = FileType.BINARY;
        FileNameExtensionFilter filter = new FileNameExtensionFilter(ft.legend(), ft.extension());
        fileChooser.setFileFilter(filter);
        switch (fileChooser.showOpenDialog(frame)) {
            case JFileChooser.CANCEL_OPTION  :
                break;
            case JFileChooser.APPROVE_OPTION :
                if (alarms.load(fileChooser.getSelectedFile())) {
            		frame.updateAlarmList();
            		frame.disableEditor();
                    frame.updateTitle();
                    frame.allowSaveItem(true);
                } else
                    JOptionPane.showMessageDialog(frame,
                                                  "Un problme est survenu pendant la lecture du fichier.",
                                                  "Alarmes non charges",
                                                  JOptionPane.ERROR_MESSAGE);
                break;
            case JFileChooser.ERROR_OPTION   :
                JOptionPane.showMessageDialog(frame,
                                              "Un problme est survenu lors du choix de fichier.",
                                              "Alarmes non charges",
                                              JOptionPane.ERROR_MESSAGE);
                break;
        }
	}
	public void saveAlarms() {
        if (!alarms.save()) {
            JOptionPane.showMessageDialog(frame,
                                          "Un problme est survenu pendant l'enregistrement.",
                                          "Alarmes non enregistres",
                                          JOptionPane.ERROR_MESSAGE);
        } else {
            frame.updateTitle();
        }
	}
	private File fileChoosedWithExtension(File file, FileNameExtensionFilter filter) {
		String[] extensions = filter.getExtensions();
		String fileName = file.getName().toLowerCase();
		if (fileName.endsWith('.' + extensions[0])) return file;
		file = new File(file.getAbsolutePath() + '.' + extensions[0]);
		return file;
	}
	public void saveAlarmsAs() {
        JFileChooser fileChooser = new JFileChooser();
        for(FileType ft : FileType.values()) {
            fileChooser.addChoosableFileFilter(new FileNameExtensionFilter(ft.legend(), ft.extension()));
        }
        fileChooser.setAcceptAllFileFilterUsed(false);
        switch (fileChooser.showSaveDialog(frame)) {
            case JFileChooser.CANCEL_OPTION  :
                break;
            case JFileChooser.APPROVE_OPTION :
                if (alarms.saveAs(fileChoosedWithExtension(fileChooser.getSelectedFile(), (FileNameExtensionFilter)fileChooser.getFileFilter()))) {
                	frame.updateTitle();
                } else {
                    JOptionPane.showMessageDialog(frame,
                                                  "Un problme est survenu pendant l'enregistrement des alarmes.",
                                                  "Alarmes non enregistres",
                                                  JOptionPane.ERROR_MESSAGE);
                }
                break;
            case JFileChooser.ERROR_OPTION   :
                JOptionPane.showMessageDialog(frame,
                                              "Un problme est survenu lors du choix de fichier pour l'enregistrement du dessin.\nLe dessin n'a pas t enregistr.",
                                              "Alarmes non enregistres",
                                              JOptionPane.ERROR_MESSAGE);
                break;
        }
	}
	public void changeAlarm(Alarm alarm, String name, String time, boolean enabled) {
		alarm.setName(name);
		if (enabled) alarm.setEnabled(); else alarm.setDisabled();
		try { alarm.setTime(time); } catch(Exception e) { }
		alarms.sort();
		alarms.setChanged();
		frame.updateAlarmList();
		frame.disableEditor();
		frame.updateTitle();
	}
	public void removeAlarm(Alarm alarm) {
		alarms.remove(alarm);
		alarms.setChanged();
		frame.updateAlarmList();
		frame.disableEditor();
		frame.updateTitle();
	}
	//en tant que WindowListener
    public void windowActivated(WindowEvent e) { }
    public void windowDeactivated(WindowEvent e) { }
    public void windowClosing(WindowEvent e) {
    	Object[] options = { "Oui", "Non" };
        if (alarms.hasChanged()) {
            switch(JOptionPane.showOptionDialog(frame,
            		                            "Des modifications ont t apportes  la liste des alarmes !\nVoulez-vous quitter sans enregistrer ?",
            		                            "Quitter Alarmes",
            		                            JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
            		                            null, options, options[1]
        									 )) {
            case JOptionPane.YES_OPTION : break;
            default                     : return;
            }
        }
        System.exit(0);
    }
    public void windowClosed(WindowEvent e) { }
    public void windowIconified(WindowEvent e) { }
    public void windowDeiconified(WindowEvent e) { }
    public void windowOpened(WindowEvent e) { }
	//en tant que Thread
	public void run() {
		while (true) {
			LocalTime now = LocalTime.now();
			for (Alarm a : alarms) {
				if (a.isEnabled()) {
					Time t = a.getTime();
					if (t.hour() == now.getHour() && t.minute() == now.getMinute()) {
						(AlarmDialog.getInstance()).display(a);
						break;
					}
				}
			}
			try { sleep(60000); } catch (InterruptedException e) { }
		}
	}
}
