package view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import controler.AlarmListListener;
import controler.Controler;
import controler.TimeControler;
import model.*;

@SuppressWarnings("serial")
public class AlarmListFrame extends JFrame {
    private static final String Title_End = " - Alerte";
    private AlarmList alarms;
    private JList<Alarm> lstAlarms;
    private JTextField txtName;
    private JTextField txtTime;
    private JCheckBox cbEnabled;
    private JButton deleteButton;
    private JButton modifyButton;
    private MenuBar menus;
    private StatusBar info;

    public AlarmListFrame(AlarmList l, Controler controler) {
    	alarms = l;
        updateTitle();
        menus = new MenuBar(controler);
        setJMenuBar(menus);

        lstAlarms = new JList<Alarm>();
        AlarmListListener alListener = new AlarmListListener(this, controler);
        lstAlarms.addListSelectionListener(alListener);
        lstAlarms.addMouseListener(alListener);
        JScrollPane listScrollPane = new JScrollPane(lstAlarms);
        listScrollPane.setPreferredSize(new Dimension(250, 80));
        add(listScrollPane, BorderLayout.CENTER);

        JPanel editAlarmPanel = new JPanel();
        editAlarmPanel.setLayout(new BoxLayout(editAlarmPanel, BoxLayout.Y_AXIS));

        Dimension dimForLabel = new Dimension(60, 20);
        Dimension dimForName = new Dimension(300, 20);
        Dimension dimForTime = new Dimension(120, 20);

        JPanel namePanel = new JPanel();
        namePanel.setLayout(new BoxLayout(namePanel, BoxLayout.X_AXIS));
        namePanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 0, 10));
        JLabel nameLabel = new JLabel("Nom :");
        nameLabel.setMaximumSize(dimForLabel);
        namePanel.add(nameLabel);
        namePanel.add(Box.createRigidArea(new Dimension(10, 0)));
        txtName = new JTextField();
        txtName.setMaximumSize(dimForName);
        namePanel.add(txtName);
        editAlarmPanel.add(namePanel);

        JPanel timeEnabledPanel = new JPanel();
        timeEnabledPanel.setLayout(new BoxLayout(timeEnabledPanel, BoxLayout.X_AXIS));
        timeEnabledPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 0, 10));
        JLabel timeLabel = new JLabel("Heure :");
        timeLabel.setMaximumSize(dimForLabel);
        timeEnabledPanel.add(timeLabel);
        timeEnabledPanel.add(Box.createRigidArea(new Dimension(10, 0)));
        txtTime = new JTextField();
        txtTime.setMaximumSize(dimForTime);
        timeEnabledPanel.add(txtTime);
        txtTime.addKeyListener(new TimeControler(txtTime));
        cbEnabled = new JCheckBox("Active");
        cbEnabled.setMaximumSize(dimForLabel);
        timeEnabledPanel.add(Box.createRigidArea(new Dimension(10, 0)));
        timeEnabledPanel.add(cbEnabled);
        editAlarmPanel.add(timeEnabledPanel);

        JPanel cmdPanel = new JPanel();
        cmdPanel.setLayout(new BoxLayout(cmdPanel, BoxLayout.LINE_AXIS));
        cmdPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        cmdPanel.add(Box.createHorizontalGlue());
        modifyButton = new JButton("Modifier");
        modifyButton.addActionListener((ActionEvent e) -> {
            Alarm alarm = lstAlarms.getSelectedValue();
            if (alarm != null) {
                controler.changeAlarm(alarm, txtName.getText(), txtTime.getText(), cbEnabled.isSelected());
            }
        });
        cmdPanel.add(modifyButton);
        cmdPanel.add(Box.createRigidArea(new Dimension(10, 0)));
        deleteButton = new JButton("Suppr.");
        deleteButton.addActionListener((ActionEvent e) -> {
            Alarm alarm = lstAlarms.getSelectedValue();
            if (alarm != null) {
            	controler.removeAlarm(alarm);
            }
        });
        cmdPanel.add(deleteButton);
        editAlarmPanel.add(cmdPanel);
        
        add(editAlarmPanel, BorderLayout.EAST);
        
        info = new StatusBar();
        add(info, BorderLayout.SOUTH);
        pack();
        //setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        addWindowListener(controler);
        setLocationRelativeTo(null);
        
        disableEditor();
    }
    public void allowSaveItem(boolean b) {
    	menus.allowSaveItem(b);
    }
    public void updateAlarmList() {
        lstAlarms.setListData(alarms.toArray(new Alarm[0]));
        info.setAlarmListSize(alarms.size());
    }
    public void updateTitle() {
        setTitle(alarms.fileName() + Title_End);
    }
    public void disableEditor() {
    	txtName.setText("");
    	txtName.setEnabled(false);
    	txtTime.setText("");
    	txtTime.setBackground(Color.WHITE);
    	txtTime.setEnabled(false);
    	cbEnabled.setSelected(false);
    	cbEnabled.setEnabled(false);
    	deleteButton.setEnabled(false);
    	modifyButton.setEnabled(false);
    }
    public void editorForSelection() {
    	Alarm a = lstAlarms.getSelectedValue();
        if (a != null) {
	    	txtName.setText(a.getName());
	    	txtName.setEnabled(true);
	    	txtTime.setText((a.getTime()).toString());
	    	txtTime.setEnabled(true);
	    	cbEnabled.setEnabled(true);
	    	cbEnabled.setSelected(a.isEnabled());
	    	deleteButton.setEnabled(true);
	    	modifyButton.setEnabled(true);
        }
    }
}
