package view;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import model.Alarm;
import model.Time;

@SuppressWarnings("serial")
public class AnalogicAlarmView extends JFrame {
	private Alarm alarm;
	private static final Dimension viewDim = new Dimension(350, 280);
	private static final Dimension timeDim = new Dimension(350, 200);
	private static final Dimension halfDim = new Dimension(350, 25);
	private static final int xc = 175;
	private static final int yc = 100;
	private static final int r1 = 90;
	private static final int r2 = 70;
	private static final int r3 = (r1 + r2) / 2;
	private static final double rpa = 40; //rayon petite aiguille
	private static final double rga = 60; //rayon grande aiguille
	private static BasicStroke thinPencil = new BasicStroke(2, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
	private static BasicStroke thickShortPencil = new BasicStroke(7, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
	private static BasicStroke thickLongPencil = new BasicStroke(4, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
	private class TimePanel extends JPanel {
		public TimePanel() {
			super();
		}
		public void paint(Graphics g) {
			int x1, y1, x2, y2;
			double angle, sinus, cosinus;
			((Graphics2D) g).setStroke(thinPencil);
			g.setColor(Color.BLACK);
			for (int i = 0; i < 12; i++) {
				angle = (2 * Math.PI) * i / 12;
				sinus = Math.sin(angle);
				cosinus = Math.cos(angle);
				x1 = xc + (int) (r1 * cosinus);
				y1 = yc - (int) (r1 * sinus);
				if (i % 3 == 0) {
					x2 = xc + (int) (r2 * cosinus);
					y2 = yc - (int) (r2 * sinus);
				} else {
					x2 = xc + (int) (r3 * cosinus);
					y2 = yc - (int) (r3 * sinus);
				}
				g.drawLine(x1, y1, x2, y2);
			}
			Time t = alarm.getTime();
			double m = t.minute();
			double h = - (t.hour() - 3 + m / 60);
			//petite aiguille
			((Graphics2D) g).setStroke(thickShortPencil);
			if (alarm.isEnabled()) g.setColor(Color.RED);
			angle = (2 * Math.PI) * h / 12;
			sinus = Math.sin(angle);
			cosinus = Math.cos(angle);
			x1 = xc;
			y1 = yc;
			x2 = xc + (int) (rpa * cosinus);
			y2 = yc - (int) (rpa * sinus);
			g.drawLine(x1, y1, x2, y2);
			//grande aiguille
			((Graphics2D) g).setStroke(thickLongPencil);
			angle = - (2 * Math.PI) * (m - 15.) / 60;
			sinus = Math.sin(angle);
			cosinus = Math.cos(angle);
			x1 = xc;
			y1 = yc;
			x2 = xc + (int) (rga * cosinus);
			y2 = yc - (int) (rga * sinus);
			g.drawLine(x1, y1, x2, y2);
		}
	}
	private TimePanel drawPanel;
	private JLabel halfLabel;
    public AnalogicAlarmView(Alarm al, AlarmListFrame frame) {
    	setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        setLocationRelativeTo(frame);
    	setTitle(al.getName());
    	alarm = al;
        setLayout(new FlowLayout());
    	drawPanel = new TimePanel();
        drawPanel.setPreferredSize(timeDim);
    	add(drawPanel);
    	halfLabel = new JLabel(al.getTime().hour()<12 ? "matin" : "aprs-midi", JLabel.CENTER);
    	halfLabel.setFont(new Font("Arial", Font.PLAIN, 20));
        halfLabel.setPreferredSize(halfDim);
    	add(halfLabel);
        setSize(viewDim); setResizable(false);
    }
}
